==> OpenCL-0.8.4.tgz: OpenCL-0.8.4/src/x917_rng.cpp <==
/*************************************************
* ANSI X9.17 RNG Source File                     *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/x917_rng.h>

namespace OpenCL {

/*************************************************
* ANSI X917 RNG Constructor                      *
*************************************************/
ANSI_X917_RNG::ANSI_X917_RNG(BlockCipher* c, u32bit n) :
   ITERATIONS_BEFORE_RESEED(n + 16),
   MAX_OUT_PER_BUFFER(c->BLOCK_SIZE / 2)
   {
   cipher = c;
   buffer.create(cipher->BLOCK_SIZE);
   state.create(cipher->BLOCK_SIZE);
   tstamp.create(cipher->BLOCK_SIZE);
   position = iteration = 0;
   reseed();
   }

/*************************************************
* ANSI X917 RNG Constructor                      *
*************************************************/
ANSI_X917_RNG::ANSI_X917_RNG(const std::string& cipher_name) :
   ITERATIONS_BEFORE_RESEED(48), MAX_OUT_PER_BUFFER(7)
   {
   cipher = 0;

   if(cipher_name != "")
      cipher = get_block_cipher(cipher_name);
   if(!cipher)
      {
      const BlockCipher* square = retrieve_block_cipher("Square");
      if(square)
         cipher = square->clone();
      }
   if(!cipher)
      cipher = get_block_cipher("TripleDES");

   if(cipher->BLOCK_SIZE <= MAX_OUT_PER_BUFFER)
      throw Internal_Error(name() + ": MAX_OUT_PER_BUFFER is too large");

   buffer.create(cipher->BLOCK_SIZE);
   state.create(cipher->BLOCK_SIZE);
   tstamp.create(cipher->BLOCK_SIZE);
   position = iteration = 0;
   reseed();
   }

/*************************************************
* Generate a random byte                         *
*************************************************/
byte ANSI_X917_RNG::random()
   {
   if(position == MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      }
   return buffer[position++];
   }

/*************************************************
* Generate a buffer of random bytes              *
*************************************************/
void ANSI_X917_RNG::randomize(byte out[], u32bit length)
   {
   while(position && length) { out[0] ^= random(); out++; length--; }
   while(length >= MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      xor_buf(out, buffer, MAX_OUT_PER_BUFFER);
      out += MAX_OUT_PER_BUFFER;
      length -= MAX_OUT_PER_BUFFER;
      }
   while(length) { out[0] ^= random(); out++; length--; }
   }

/*************************************************
* Refill the internal state                      *
*************************************************/
void ANSI_X917_RNG::generate(u64bit input)
   {
   xor_buf(tstamp, (byte*)&input, 8);
   cipher->encrypt(tstamp);
   xor_buf(buffer, state, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(buffer);
   xor_buf(state, buffer, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(state);
   position = 0;
   }

/*************************************************
* Add entropy to internal state                  *
*************************************************/
void ANSI_X917_RNG::add_randomness(const byte data[], u32bit length) throw()
   {
   while(length)
      {
      u32bit added = std::min(state.size(), length);
      xor_buf(state, data, added);
      generate(system_clock());
      length -= added;
      data += added;
      }
   reseed();
   }

/*************************************************
* Reseed the internal state                      *
*************************************************/
void ANSI_X917_RNG::reseed()
   {
   SecureVector<byte> key(cipher->MAXIMUM_KEYLENGTH);
   generate(system_clock());
   state = buffer;
   generate(system_clock());
   tstamp = buffer;
   randomize(key, key.size());
   cipher->set_key(key, key.size());
   generate(system_time());
   iteration = 0;
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void ANSI_X917_RNG::clear() throw()
   {
   cipher->clear();
   buffer.clear();
   tstamp.clear();
   state.clear();
   position = iteration = 0;
   }

/*************************************************
* Return the name of this type                   *
*************************************************/
std::string ANSI_X917_RNG::name() const
   {
   return "ANSI X9.17(" + cipher->name() + ") RNG";
   }

}

==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/src/x917_rng.cpp <==
/*************************************************
* ANSI X9.17 RNG Source File                     *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/x917_rng.h>
#include <opencl/lookup.h>

#include <stdio.h>

namespace OpenCL {

/*************************************************
* ANSI X917 RNG Constructor                      *
*************************************************/
ANSI_X917_RNG::ANSI_X917_RNG(BlockCipher* c, u32bit n) :
   ITERATIONS_BEFORE_RESEED(n + 16),
   MAX_OUT_PER_BUFFER(c->BLOCK_SIZE / 2)
   {
   printf("WARNING: This constructor of ANSI_X917_RNG is deprecated; "
          "(read doc/deprecated.txt for details)\n");

   cipher = c;
   buffer.create(cipher->BLOCK_SIZE);
   state.create(cipher->BLOCK_SIZE);
   tstamp.create(cipher->BLOCK_SIZE);
   position = iteration = 0;
   reseed();
   }

/*************************************************
* ANSI X917 RNG Constructor                      *
*************************************************/
ANSI_X917_RNG::ANSI_X917_RNG(const std::string& cipher_name) :
   ITERATIONS_BEFORE_RESEED(48), MAX_OUT_PER_BUFFER(8)
   {
   if(cipher_name != "")
      printf("WARNING: Passing an argument (%s) to this constructor of\n"
             "ANSI_X917_RNG is deprecated; (read doc/deprecated.txt for "
             "details)\n", cipher_name.c_str());

   cipher = get_block_cipher("AES");

   buffer.create(cipher->BLOCK_SIZE);
   state.create(cipher->BLOCK_SIZE);
   tstamp.create(cipher->BLOCK_SIZE);
   position = iteration = 0;
   reseed();
   }

/*************************************************
* Generate a random byte                         *
*************************************************/
byte ANSI_X917_RNG::random()
   {
   if(position == MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      }
   return buffer[position++];
   }

/*************************************************
* Generate a buffer of random bytes              *
*************************************************/
void ANSI_X917_RNG::randomize(byte out[], u32bit length)
   {
   while(position && length) { out[0] ^= random(); out++; length--; }
   while(length >= MAX_OUT_PER_BUFFER)
      {
      generate(system_clock());
      if(++iteration == ITERATIONS_BEFORE_RESEED)
         reseed();
      xor_buf(out, buffer, MAX_OUT_PER_BUFFER);
      out += MAX_OUT_PER_BUFFER;
      length -= MAX_OUT_PER_BUFFER;
      }
   while(length) { out[0] ^= random(); out++; length--; }
   }

/*************************************************
* Refill the internal state                      *
*************************************************/
void ANSI_X917_RNG::generate(u64bit input)
   {
   xor_buf(tstamp, (byte*)&input, 8);
   cipher->encrypt(tstamp);
   xor_buf(buffer, state, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(buffer);
   xor_buf(state, buffer, tstamp, cipher->BLOCK_SIZE);
   cipher->encrypt(state);
   position = 0;
   }

/*************************************************
* Add entropy to internal state                  *
*************************************************/
void ANSI_X917_RNG::add_randomness(const byte data[], u32bit length) throw()
   {
   while(length)
      {
      u32bit added = std::min(state.size(), length);
      xor_buf(state, data, added);
      generate(system_clock());
      length -= added;
      data += added;
      }
   reseed();
   }

/*************************************************
* Reseed the internal state                      *
*************************************************/
void ANSI_X917_RNG::reseed()
   {
   SecureVector<byte> key(cipher->MAXIMUM_KEYLENGTH);
   generate(system_clock());
   state = buffer;
   generate(system_clock());
   tstamp = buffer;
   randomize(key, key.size());
   cipher->set_key(key, key.size());
   generate(system_time());
   iteration = 0;
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void ANSI_X917_RNG::clear() throw()
   {
   cipher->clear();
   buffer.clear();
   tstamp.clear();
   state.clear();
   position = iteration = 0;
   }

/*************************************************
* Return the name of this type                   *
*************************************************/
std::string ANSI_X917_RNG::name() const
   {
   return "X9.17(" + cipher->name() + ")";
   }

}
